@@ -1,26 +1,28 @@
+// $OpenLDAP$
 /*
- * Copyright 2000, OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 2000-2016 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
 #ifndef LDAP_ENTRY_LIST_H
 #define LDAP_ENTRY_LIST_H
 
+#include <cstdio>
 #include <list>
 
 class LDAPEntry;
    
-typedef list<LDAPEntry> EntryList;
-
 /**
  * For internal use only.
  * 
- * This class is used by LDAPSearchResults to store a list of
+ * This class is used by LDAPSearchResults to store a std::list of
  * LDAPEntry-Objects
  */
 class LDAPEntryList{
+    typedef std::list<LDAPEntry> ListType;
+
     public:
-	typedef EntryList::const_iterator const_iterator;
+	typedef ListType::const_iterator const_iterator;
 
         /**
          * Copy-Constructor
@@ -42,6 +44,11 @@ class LDAPEntryList{
          */
         size_t size() const;
 
+        /**
+         * @return true if there are zero entries currently stored in the list.
+         */
+        bool empty() const;
+
         /**
          * @return An iterator pointing to the first element of the list.
          */
@@ -58,6 +65,6 @@ class LDAPEntryList{
         void addEntry(const LDAPEntry& e);
 
     private:
-        EntryList m_entries;
+        ListType m_entries;
 };
 #endif // LDAP_ENTRY_LIST_H
