@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/schema-map.c,v 1.59.2.10 2010/04/13 20:23:43 kurt Exp $ */
+/* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2010 The OpenLDAP Foundation.
+ * Copyright 1999-2016 The OpenLDAP Foundation.
  * Portions Copyright 1999 Dmitry Kovalev.
  * Portions Copyright 2002 Pierangelo Masarati.
  * Portions Copyright 2004 Mark Adamson.
@@ -160,7 +160,12 @@ backsql_make_attr_query(
 	at_map->bam_query = bb.bb_val.bv_val;
 
 #ifdef BACKSQL_COUNTQUERY
-	/* Query to count how many rows will be returned. */
+	/* Query to count how many rows will be returned.
+
+	SELECT COUNT(*) FROM <from_tbls> WHERE <keytbl>.<keycol>=?
+		[ AND <join_where> ]
+
+	 */
 	BER_BVZERO( &bb.bb_val );
 	bb.bb_len = 0;
 	backsql_strfcat_x( &bb, NULL, "lblbcbl", 
@@ -194,7 +199,7 @@ backsql_add_sysmaps( backsql_info *bi, backsql_oc_map_rec *oc_map )
 	struct berbuf		bb;
 	
 	sbv.bv_val = s;
-	sbv.bv_len = snprintf( s, sizeof( s ), "%ld", oc_map->bom_id );
+	sbv.bv_len = snprintf( s, sizeof( s ), BACKSQL_IDNUMFMT, oc_map->bom_id );
 
 	/* extra objectClasses */
 	at_map = (backsql_at_map_rec *)ch_calloc(1, 
@@ -237,7 +242,7 @@ backsql_add_sysmaps( backsql_info *bi, backsql_oc_map_rec *oc_map )
 			"INSERT INTO ldap_entry_objclasses "
 			"(entry_id,oc_name) VALUES "
 			"((SELECT id FROM ldap_entries "
-			"WHERE oc_map_id=%lu "
+			"WHERE oc_map_id=" BACKSQL_IDNUMFMT " "
 			"AND keyval=?),?)", oc_map->bom_id );
 		at_map->bam_add_proc = ch_strdup( tmp );
 	}
@@ -251,7 +256,7 @@ backsql_add_sysmaps( backsql_info *bi, backsql_oc_map_rec *oc_map )
 		snprintf( tmp, sizeof(tmp), 
 			"DELETE FROM ldap_entry_objclasses "
 			"WHERE entry_id=(SELECT id FROM ldap_entries "
-			"WHERE oc_map_id=%lu "
+			"WHERE oc_map_id=" BACKSQL_IDNUMFMT " "
 			"AND keyval=?) AND oc_name=?",
 			oc_map->bom_id );
 		at_map->bam_delete_proc = ch_strdup( tmp );
@@ -292,7 +297,7 @@ struct backsql_attr_schema_info {
 	backsql_info	*bas_bi;
 	SQLHDBC		bas_dbh;
 	SQLHSTMT	bas_sth;
-	unsigned long	*bas_oc_id;
+	backsql_key_t	*bas_oc_id;
 	int		bas_rc;
 };
 
@@ -312,7 +317,7 @@ backsql_oc_get_attr_mapping( void *v_oc, void *v_bas )
 		"executing at_query\n"
 		"    \"%s\"\n"
 		"    for objectClass \"%s\"\n"
-		"    with param oc_id=\"%lu\"\n",
+		"    with param oc_id=" BACKSQL_IDNUMFMT "\n",
 		bas->bas_bi->sql_at_query,
 		BACKSQL_OC_NAME( oc_map ),
 		*bas->bas_oc_id );
@@ -323,7 +328,7 @@ backsql_oc_get_attr_mapping( void *v_oc, void *v_bas )
 			"error executing at_query\n"
 			"    \"%s\"\n"
 			"    for objectClass \"%s\"\n"
-			"    with param oc_id=\"%lu\"\n",
+			"    with param oc_id=" BACKSQL_IDNUMFMT "\n",
 			bas->bas_bi->sql_at_query,
 			BACKSQL_OC_NAME( oc_map ),
 			*bas->bas_oc_id );
@@ -416,6 +421,7 @@ backsql_oc_get_attr_mapping( void *v_oc, void *v_bas )
 				Debug( LDAP_DEBUG_TRACE, "backsql_oc_get_attr_mapping(): "
 					"unable to fetch attribute \"%s\": %s (%d)\n",
 					buf, text, rc );
+				ch_free( at_map );
 				return BACKSQL_AVL_STOP;
 			}
 		}
@@ -464,6 +470,7 @@ backsql_oc_get_attr_mapping( void *v_oc, void *v_bas )
 					"in objectClass \"%s\" map\n",
 					at_map->bam_ad->ad_cname.bv_val,
 					oc_map->bom_oc->soc_cname.bv_val, 0 );
+			ch_free( at_map );
 		}
 
 		if ( !BER_BVISNULL( &bas->bas_bi->sql_upper_func ) &&
@@ -498,7 +505,7 @@ backsql_load_schema_map( backsql_info *bi, SQLHDBC dbh )
 	SQLHSTMT 			sth = SQL_NULL_HSTMT;
 	RETCODE				rc;
 	BACKSQL_ROW_NTS			oc_row;
-	unsigned long			oc_id;
+	backsql_key_t			oc_id;
 	backsql_oc_map_rec		*oc_map;
 	struct backsql_attr_schema_info	bas;
 
@@ -564,11 +571,13 @@ backsql_load_schema_map( backsql_info *bi, SQLHDBC dbh )
 				{ 1, "name" },
 				{ 2, "keytbl" },
 				{ 3, "keycol" },
-				{ delete_proc_idx + 1, "expect_return" },
+				{ -1, "expect_return" },
 				{ -1, NULL },
 			};
 			int i;
 
+			required[4].idx = delete_proc_idx + 1;
+
 			for ( i = 0; required[ i ].name != NULL; i++ ) {
 				if ( oc_row.value_len[ required[ i ].idx ] <= 0 ) {
 					Debug( LDAP_DEBUG_ANY,
@@ -609,10 +618,11 @@ backsql_load_schema_map( backsql_info *bi, SQLHDBC dbh )
 		oc_map = (backsql_oc_map_rec *)ch_calloc( 1,
 				sizeof( backsql_oc_map_rec ) );
 
-		if ( lutil_atoulx( &oc_map->bom_id, oc_row.cols[ 0 ], 0 ) != 0 ) {
+		if ( BACKSQL_STR2ID( &oc_map->bom_id, oc_row.cols[ 0 ], 0 ) != 0 ) {
 			Debug( LDAP_DEBUG_TRACE, "backsql_load_schema_map(): "
 				"unable to parse id=\"%s\"\n", 
 				oc_row.cols[ 0 ], 0, 0 );
+			ch_free( oc_map );
 			return LDAP_OTHER;
 		}
 
@@ -621,6 +631,7 @@ backsql_load_schema_map( backsql_info *bi, SQLHDBC dbh )
 			Debug( LDAP_DEBUG_TRACE, "backsql_load_schema_map(): "
 				"objectClass \"%s\" is not defined in schema\n", 
 				oc_row.cols[ 1 ], 0, 0 );
+			ch_free( oc_map );
 			return LDAP_OTHER;	/* undefined objectClass ? */
 		}
 		
@@ -639,6 +650,7 @@ backsql_load_schema_map( backsql_info *bi, SQLHDBC dbh )
 			Debug( LDAP_DEBUG_TRACE, "backsql_load_schema_map(): "
 				"unable to parse expect_return=\"%s\" for objectClass \"%s\"\n", 
 				oc_row.cols[ delete_proc_idx + 1 ], oc_row.cols[ 1 ], 0 );
+			ch_free( oc_map );
 			return LDAP_OTHER;
 		}
 
@@ -659,6 +671,7 @@ backsql_load_schema_map( backsql_info *bi, SQLHDBC dbh )
 						text, rc );
 				backsql_PrintErrors( bi->sql_db_env, dbh,
 						sth, rc );
+				ch_free( oc_map );
 				return LDAP_OTHER;
 			}
 		}
@@ -673,6 +686,7 @@ backsql_load_schema_map( backsql_info *bi, SQLHDBC dbh )
 			Debug( LDAP_DEBUG_TRACE, "backsql_load_schema_map(): "
 					"duplicate objectClass \"%s\" in objectClass map\n",
 					oc_map->bom_oc->soc_cname.bv_val, 0, 0 );
+			ch_free( oc_map );
 			return LDAP_OTHER;
 		}
 		if ( avl_insert( &bi->sql_oc_by_id, oc_map, backsql_cmp_oc_id, avl_dup_error ) == -1 ) {
@@ -725,7 +739,7 @@ backsql_load_schema_map( backsql_info *bi, SQLHDBC dbh )
 		return LDAP_OTHER;
 	}
 
-	rc = backsql_BindParamInt( sth, 1, SQL_PARAM_INPUT, &oc_id );
+	rc = backsql_BindParamNumID( sth, 1, SQL_PARAM_INPUT, &oc_id );
 	if ( rc != SQL_SUCCESS ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_load_schema_map(): "
 			"error binding param \"oc_id\" for at_query\n", 0, 0, 0 );
@@ -817,7 +831,7 @@ backsql_id2oc( backsql_info *bi, unsigned long id )
  
 #ifdef BACKSQL_TRACE
 	Debug( LDAP_DEBUG_TRACE, "==>oc_with_id(): "
-		"searching for objectclass with id='%d'\n", id, 0, 0 );
+		"searching for objectclass with id=%lu\n", id, 0, 0 );
 #endif /* BACKSQL_TRACE */
 
 	tmp.bom_id = id;
@@ -826,12 +840,12 @@ backsql_id2oc( backsql_info *bi, unsigned long id )
 
 #ifdef BACKSQL_TRACE
 	if ( res != NULL ) {
-		Debug( LDAP_DEBUG_TRACE, "<==oc_with_name(): "
-			"found name=\"%s\", id=%d\n",
+		Debug( LDAP_DEBUG_TRACE, "<==oc_with_id(): "
+			"found name=\"%s\", id=%lu\n",
 			BACKSQL_OC_NAME( res ), res->bom_id, 0 );
 	} else {
-		Debug( LDAP_DEBUG_TRACE, "<==oc_with_name(): "
-			"not found\n", 0, 0, 0 );
+		Debug( LDAP_DEBUG_TRACE, "<==oc_with_id(): "
+			"id=%lu not found\n", res->bom_id, 0, 0 );
 	}
 #endif /* BACKSQL_TRACE */
 	
